__version__ = "4.2.1"

# __version__ has to be define in the first line

"""
pysal.lib: Python Spatial Analysis Library (core)
================================================


Documentation
-------------
PySAL documentation is available in two forms: python docstrings and an html \
        webpage at http://pysal.org/

Available sub-packages
----------------------

cg
    Basic data structures and tools for Computational Geometry
examples
    Example data sets for testing and documentation
io
    Basic functions used by several sub-packages
weights
    Tools for creating and manipulating weights
"""

from libpysal import cg
from libpysal import io
from libpysal import weights
from libpysal import examples




from spint.gravity import Gravity, Production, Attraction, Doubly
from spint.utils import CPC, sorensen, srmse
from spint.vec_SA import VecMoran as Moran_Vector
from spint.dispersion import phi_disp, alpha_disp



from esda.moran import (
    Moran,
    Moran_BV,
    Moran_BV_matrix,
    Moran_Local,
    Moran_Local_BV,
    Moran_Rate,
    Moran_Local_Rate,
)
from esda.getisord import G, G_Local
from esda.geary import Geary
from esda.join_counts import Join_Counts
from esda.gamma import Gamma
from esda.util import fdr
from esda.smaup import Smaup
from esda.lee import Spatial_Pearson, Local_Spatial_Pearson
from esda.silhouettes import (path_silhouette, boundary_silhouette,
                          silhouette_alist, nearest_label)



import copy
import sys
import time

# external imports
import numpy as np
import numpy.linalg as la

import scipy as sp
import scipy.stats as stats
from libpysal.cg.kdtree import KDTree
from scipy.spatial.distance import pdist, cdist

import pandas

try:
    from patsy import PatsyError
except ImportError:
    PatsyError = Exception

RTOL = .00001
ATOL = 1e-7
MISSINGVALUE = None

######################
# Decorators/Utils   #
######################

# import numba.jit OR create mimic decorator and set existence flag
try:
    from numba import jit
    HAS_JIT = True
except ImportError:
    def jit(function=None, **kwargs):
        """Mimic numba.jit() with synthetic wrapper
        """
        if function is not None:
            def wrapped(*original_args, **original_kw):
                """Case 1 - structure of a standard decorator
                i.e., jit(function)(*args, **kwargs)
                """
                return function(*original_args, **original_kw)
            return wrapped
        else:
            def partial_inner(func):
                """Case 2 - returns Case 1
                i.e., jit()(function)(*args, **kwargs)
                """
                return jit(func)
            return partial_inner
    HAS_JIT = False

def simport(modname):
    """
    Safely import a module without raising an error.

    Parameters
    -----------
    modname : str
              module name needed to import
    
    Returns
    --------
    tuple of (True, Module) or (False, None) depending on whether the import
    succeeded.

    Notes
    ------
    Wrapping this function around an iterative context or a with context would
    allow the module to be used without necessarily attaching it permanently in
    the global namespace:

    
        for t,mod in simport('pandas'):
            if t:
                mod.DataFrame()
            else:
                #do alternative behavior here
            del mod #or don't del, your call

    instead of:

        t, mod = simport('pandas')
        if t:
            mod.DataFrame()
        else:
            #do alternative behavior here

    The first idiom makes it work kind of a like a with statement.
    """
    try:
        exec('import {}'.format(modname))
        return True, eval(modname)
    except:
        return False, None

def requires(*args, **kwargs):
    """
    Decorator to wrap functions with extra dependencies:

    Arguments
    ---------
    args : list
            list of strings containing module to import
    verbose : bool
                boolean describing whether to print a warning message on import
                failure
    Returns
    -------
    Original function is all arg in args are importable, otherwise returns a
    function that passes.
    """
    v = kwargs.pop('verbose', True)
    wanted = copy.deepcopy(args)
    def inner(function):
        available = [simport(arg)[0] for arg in args]
        if all(available):
            return function
        else:
            def passer(*args,**kwargs):
                if v:
                    missing = [arg for i, arg in enumerate(wanted) if not available[i]]
                    print(('missing dependencies: {d}'.format(d=missing)))
                    print(('not running {}'.format(function.__name__)))
                else:
                    pass
            return passer
    return inner



# coding: utf-8
import codecs
import os.path
import os
import versioneer
from setuptools import setup, find_packages
from distutils.command.build_py import build_py

with open('README.md') as file:
    long_description = file.read()


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


print(find_packages())
# BEFORE importing distutils, remove MANIFEST. distutils doesn't properly
# update it when the contents of directories change.
if os.path.exists('MANIFEST'):
    os.remove('MANIFEST')

def _get_requirements_from_files(groups_files):
    groups_reqlist = {}

    for k,v in groups_files.items():
        with open(v, 'r') as f:
            pkg_list = f.read().splitlines()
        groups_reqlist[k] = pkg_list

    return groups_reqlist

def setup_package():
    # get all file endings and copy whole file names without a file suffix
    # assumes nested directories are only down one level
    _groups_files = {
        'base': 'requirements.txt',
        'plus': 'requirements_plus.txt',
        'dev': 'requirements_dev.txt',
        'docs': 'requirements_docs.txt'
    }

    reqs = _get_requirements_from_files(_groups_files)
    install_reqs = reqs.pop('base')
    extras_reqs = reqs
    setup(
        name='pysal',
        version=versioneer.get_version(),
        cmdclass=versioneer.get_cmdclass({"build_py": build_py}),
        description="A library of spatial analysis functions.",
        long_description=long_description,
        long_description_content_type="text/markdown",
        maintainer="PySAL Developers",
        maintainer_email='pysal-dev@googlegroups.com',
        url='http://pysal.org',
        download_url='https://pypi.python.org/pypi/pysal',
        license='BSD',
        packages=find_packages(),
        python_requires='>3.6',
        test_suite='nose.collector',
        tests_require=['nose'],
        keywords='spatial statistics',
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Science/Research',
            'Intended Audience :: Developers',
            'Intended Audience :: Education',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: GIS',
            'License :: OSI Approved :: BSD License',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8'
        ],
        install_requires=install_reqs,
        extras_require=extras_reqs,
    )


if __name__ == '__main__':
    setup_package()
